@@ -1,3 +1,4 @@
+from collections import defaultdict
 from lingtrain_aligner import helper, preprocessor
 import json
 import pathlib
@@ -14,6 +15,16 @@ MARKS = [preprocessor.H1, preprocessor.H2, preprocessor.H3,
          preprocessor.QUOTE_TEXT, preprocessor.QUOTE_NAME, preprocessor.IMAGE]
 
 
+def is_empty_cells(db_path):
+    index = helper.get_flatten_doc_index(db_path)
+    for item in index:
+        from_ids = json.loads(item[0][1])
+        to_ids = json.loads(item[0][3])
+        if len(from_ids) == 0 or len(to_ids) == 0:
+            return True
+    return False
+    
+
 def get_paragraphs(db_path, direction="from"):
     """Read all paragraphs with marks from database"""
     # default direction is 'from'
@@ -23,13 +34,19 @@ def get_paragraphs(db_path, direction="from"):
     index = helper.get_flatten_doc_index(db_path)
     page = list(zip(index, range(len(index))))
 
-    data, _, __ = helper.get_doc_items(page, db_path)
+    data, _, __ = helper.get_doc_items(page, db_path)    
+    lang_from, lang_to = helper.get_lang_codes(db_path)
 
     # extract paragraph info
     from_ids, to_ids = set(), set()
+    sent_counter_dict = defaultdict(int)
     for item in index:
-        from_ids.update(json.loads(item[0][1]))
-        to_ids.update(json.loads(item[0][3]))
+        from_json = json.loads(item[0][1])
+        to_json = json.loads(item[0][3])
+        from_ids.update(from_json)
+        to_ids.update(to_json)
+        sent_counter_dict[lang_from] += len(from_json)
+        sent_counter_dict[lang_to] += len(to_json)
 
     splitted_from = helper.get_splitted_from_by_id(db_path, from_ids)
     splitted_to = helper.get_splitted_to_by_id(db_path, to_ids)
@@ -38,85 +55,26 @@ def get_paragraphs(db_path, direction="from"):
     paragraphs_to_dict = helper.get_paragraph_dict(splitted_to)
 
     meta = helper.get_meta_dict(db_path)
+    meta_dict, paragraphs_dict = dict(), dict()
+    meta_dict[lang_from] = prepare_meta(meta, "from")
+    meta_dict[lang_to] = prepare_meta(meta, "to")
 
-    paragraphs_from, paragraphs_to = [], []
-    if direction == "from":
-        prev_meta = paragraphs_from_dict[json.loads(index[0][0][1])[0]]
-    else:
-        prev_meta = paragraphs_to_dict[json.loads(index[0][0][3])[0]]
+    main_lang_code = lang_from if direction == "from" else lang_to
 
-    prev_paragraph = prev_meta[0]
-    prev_h1, prev_h2, prev_h3, prev_h4, prev_h5, prev_di = prev_meta[
-        1], prev_meta[2], prev_meta[3], prev_meta[4], prev_meta[5], prev_meta[6]
+    gen_main = get_next_paragraph(
+        index, data, paragraphs_from_dict, paragraphs_to_dict, direction)
+    par_info = [(par_from, par_to, par_id)
+                for par_from, par_to, par_id, _, _ in gen_main]
+    par_info_list = list(zip(*par_info))
 
-    curr_from, curr_to = [data[0]["text_from"]], [data[0]["text_to"]]
+    paragraphs_from, paragraphs_to, par_ids = par_info_list[0], par_info_list[1], par_info_list[2]
 
-    for item, texts in zip(index[1:], data[1:]):
-        fid = max(json.loads(item[0][1]))
-        tid = max(json.loads(item[0][3]))
+    paragraphs_dict[lang_from] = paragraphs_from
+    paragraphs_dict[lang_to] = paragraphs_to
 
-        if direction == "from":
-            curr_paragraph = paragraphs_from_dict[fid][0]
-        else:
-            curr_paragraph = paragraphs_to_dict[tid][0]
+    meta_info = {"items": meta_dict, "main_lang_code": main_lang_code}
 
-        # print("item", item)
-        # print("fid", fid)
-        # print("tid", tid)
-        # print("prev_paragraph", prev_paragraph)
-        # print("curr_paragraph", curr_paragraph)
-
-        curr_h1 = paragraphs_from_dict[fid][1] if direction == "from" else paragraphs_to_dict[tid][1]
-        curr_h2 = paragraphs_from_dict[fid][2] if direction == "from" else paragraphs_to_dict[tid][2]
-        curr_h3 = paragraphs_from_dict[fid][3] if direction == "from" else paragraphs_to_dict[tid][3]
-        curr_h4 = paragraphs_from_dict[fid][4] if direction == "from" else paragraphs_to_dict[tid][4]
-        curr_h5 = paragraphs_from_dict[fid][5] if direction == "from" else paragraphs_to_dict[tid][5]
-        curr_di = paragraphs_from_dict[fid][6] if direction == "from" else paragraphs_to_dict[tid][6]
-
-        if curr_paragraph == prev_paragraph:
-            curr_from.append(texts["text_from"])
-            curr_to.append(texts["text_to"])
-        else:
-            paragraphs_from.append(curr_from)
-            paragraphs_to.append(curr_to)
-
-            prev_paragraph = curr_paragraph
-            curr_from, curr_to = [texts["text_from"]], [texts["text_to"]]
-
-        if curr_h1 != prev_h1:
-            paragraphs_from.append(H1_MARK)
-            paragraphs_to.append(curr_h1-1)
-            prev_h1 = curr_h1
-
-        if curr_h2 != prev_h2:
-            paragraphs_from.append(H2_MARK)
-            paragraphs_to.append(curr_h2-1)
-            prev_h2 = curr_h2
-
-        if curr_h3 != prev_h3:
-            paragraphs_from.append(H3_MARK)
-            paragraphs_to.append(curr_h3-1)
-            prev_h3 = curr_h3
-
-        if curr_h4 != prev_h4:
-            paragraphs_from.append(H4_MARK)
-            paragraphs_to.append(curr_h4-1)
-            prev_h4 = curr_h4
-
-        if curr_h5 != prev_h5:
-            paragraphs_from.append(H5_MARK)
-            paragraphs_to.append(curr_h5-1)
-            prev_h5 = curr_h5
-
-        if curr_di != prev_di:
-            paragraphs_from.append(DIVIDER_MARK)
-            paragraphs_to.append(curr_di-1)
-            prev_di = curr_di
-
-    paragraphs_from.append(curr_from)
-    paragraphs_to.append(curr_to)
-
-    return paragraphs_from, paragraphs_to, meta
+    return paragraphs_dict, par_ids, meta_info, sent_counter_dict
 
 
 def get_paragraphs_polybook(db_paths, direction="to", par_amount=0):
@@ -322,12 +280,6 @@ def merge_sent_mappings(sent_mapping_dict, lang_codes, target_language_side, mai
 
 def merge_sentences_mapping(sent_mapping_dict, par_ids):
     """Merge sentences mapping according to the weakest paragraph mapping (par_ids)"""
-
-    # print("par_ids", par_ids[:100])
-    # print("len(par_ids)", len(par_ids))
-
-    # return
-
     for target_lang_code in sent_mapping_dict:
         for lang in sent_mapping_dict[target_lang_code]:
             res = [[] for _ in range(len(par_ids))]
@@ -335,8 +287,6 @@ def merge_sentences_mapping(sent_mapping_dict, par_ids):
             curr_par_id = 1
             res[0].extend(sent_mapping_dict[target_lang_code][lang][0][1])
 
-            # len(sent_mapping_dict[target_lang_code][lang]) > curr_par_id and
-
             for i, par_id in enumerate(par_ids[1:]):
                 while len(sent_mapping_dict[target_lang_code][lang]) > curr_par_id and sent_mapping_dict[target_lang_code][lang][curr_par_id][0] <= par_id:
 
@@ -347,11 +297,6 @@ def merge_sentences_mapping(sent_mapping_dict, par_ids):
                         res[i+1].extend(sent_mapping_dict[target_lang_code]
                                         [lang][curr_par_id][1])
 
-                    # if target_lang_code == "uk" and curr_par_id >=47 and curr_par_id < 49:
-                    #     print("par_id", par_id)
-                    #     print("curr_par_id", sent_mapping_dict[target_lang_code][lang][curr_par_id][0])
-                    #     print("res", res[46:49])
-
                     curr_par_id += 1
 
             # if target_lang_code == "uk" and lang == "ru":
@@ -362,12 +307,9 @@ def merge_sentences_mapping(sent_mapping_dict, par_ids):
 
 def merge_sub_arrays(res, curr, a, b, left, right, len1, len2, to_append):
     """Merge subarrays"""
-
     # a =  [[6,  7], [8], [9,  10, 11, 12],[13,  14,15],[16]]
     # b =  [[6],[7], [8], [9],[10, 11, 12,  13],[14,15,  16]]
-
     # res = [[6, 7], [8], [9, 10, 11, 12, 13, 14, 15, 16]]
-
     if len1 == len2:
         curr.extend(to_append)
         res.append(curr)
@@ -434,8 +376,6 @@ def get_next_paragraph(index, data, paragraphs_from_dict, paragraphs_to_dict, di
 
     prev_paragraph_max = prev_meta_max[0]
 
-    # prev_h1, prev_h2, prev_h3, prev_h4, prev_h5, prev_di = prev_meta[1], prev_meta[2], prev_meta[3], prev_meta[4], prev_meta[5], prev_meta[6]
-
     curr_from, curr_to = [data[0]["text_from"]], [data[0]["text_to"]]
     curr_splitted_ids_from, curr_splitted_ids_to = [json.loads(index[0][0][1])], [
         json.loads(index[0][0][3])]
@@ -453,21 +393,12 @@ def get_next_paragraph(index, data, paragraphs_from_dict, paragraphs_to_dict, di
             curr_paragraph_min = paragraphs_to_dict[tid_min][0]
             curr_paragraph_max = paragraphs_to_dict[tid_max][0]
 
-        # if curr_paragraph_min > 63 and curr_paragraph_min < 67:
-        #     print("tid_min, tid_max", tid_min, tid_max)
-        #     print("curr_paragraph_min", "curr_paragraph_max", curr_paragraph_min, curr_paragraph_max)
-        #     print("prev_paragraph_max", prev_paragraph_max)
-
         if curr_paragraph_min == prev_paragraph_max:
-
             curr_from.append(texts["text_from"])
             curr_to.append(texts["text_to"])
             curr_splitted_ids_from.append(json.loads(item[0][1]))
             curr_splitted_ids_to.append(json.loads(item[0][3]))
 
-            # if curr_paragraph_min == 65 and curr_paragraph_max == 66:
-            #     print(curr_from)
-
             prev_paragraph_max = curr_paragraph_max
 
         else:
@@ -481,7 +412,7 @@ def get_next_paragraph(index, data, paragraphs_from_dict, paragraphs_to_dict, di
     yield curr_from, curr_to, prev_paragraph_max, curr_splitted_ids_from, curr_splitted_ids_to
 
 
-def create_book(paragraphs_from, paragraphs_to, meta, output_path, template, styles=[]):
+def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, output_path, template, styles=[]):
     """Generate html"""
     # ensure path is existed
     pathlib.Path(output_path).parent.mkdir(parents=True, exist_ok=True)
@@ -495,164 +426,83 @@ def create_book(paragraphs_from, paragraphs_to, meta, output_path, template, sty
     else:
         css = generate_css([])
         sent_cycle = 2
+    
+    min_par_len = min([len(paragraphs[x]) for x in paragraphs])
+    min_par_len = min(min_par_len, len(delimeters))
+
+    header_text = ""
+    for i, lang in enumerate(lang_ordered):
+        header_text += f"{sent_counter[lang]} sent. [{lang}] "
+        if i == 0:
+            header_text += "• "
 
     with open(output_path, "w", encoding="utf8") as res_html:
         # --------------------HEAD
         res_html.write(f"""
-        <html><head>
-            <link rel="stylesheet" href="main.css">
-            <link rel="preconnect" href="https://fonts.gstatic.com">
-            <link href="https://fonts.googleapis.com/css2?family=Noto+Serif:wght@400&display=swap" rel="stylesheet">
-            <link href="https://fonts.googleapis.com/css2?family=Oswald:wght@300;400&display=swap" rel="stylesheet">
-            <title>Lingtrain Magic Book</title>
-            {css}
-        </head>
-        <body>""")
+<html><head>
+    <link rel="preconnect" href="https://fonts.gstatic.com">
+    <link href="https://fonts.googleapis.com/css2?family=Noto+Serif:wght@400&display=swap" rel="stylesheet">
+    <link href="https://fonts.googleapis.com/css2?family=Oswald:wght@300;400&display=swap" rel="stylesheet">
+    <link href="https://fonts.googleapis.com/css2?family=Josefin+Sans&display=swap" rel="stylesheet">
+    <title>Lingtrain Magic Book</title>
+    {css}
+</head>
+<body>
+<div class="lt-header">🚀 lingtrain parallel book 🡒 {header_text} 🡒 {min_par_len} paragpaphs</div>""")
 
         # --------------------BOOK
         res_html.write("<div class='dt cont'>")
 
-        #--------------------TITLE and AUTHOR
-        res_html.write("<div class='dt-row header'><div class='par dt-cell'>")
-
-        title_from = get_meta_from(meta, preprocessor.TITLE)
-        if title_from:
-            res_html.write("<h1>" + title_from[0] + "</h1>")
-        author_from = get_meta_from(meta, preprocessor.AUTHOR)
-        if author_from:
-            res_html.write("<h2>" + author_from[0] + "</h2>")
-
-        res_html.write("</div><div class='par dt-cell'>")
-
-        title_to = get_meta_to(meta, preprocessor.TITLE)
-        if title_to:
-            res_html.write("<h1>" + title_to[0] + "</h1>")
-        author_to = get_meta_to(meta, preprocessor.AUTHOR)
-        if author_to:
-            res_html.write("<h2>" + author_to[0] + "</h2>")
-
-        res_html.write("</div></div>")
-
         # --------------------DIVIDER
         res_html.write("<div class='dt-row header'>")
-        res_html.write(
-            "<div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div><div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div>")
+        for _ in range(len(lang_ordered)):
+            res_html.write(
+                f"<div class='dt-cell divider'><img class='divider-img' src='{DIVIDER_URL}'/></div>")
         res_html.write("</div>")
 
-        # --------------------FIRST HEADERS IF EXIST
-        write_header(res_html, meta, preprocessor.H1, occurence=0)
-        write_header(res_html, meta, preprocessor.H2, occurence=0)
-        write_header(res_html, meta, preprocessor.H3, occurence=0)
-        write_header(res_html, meta, preprocessor.H4, occurence=0)
-        write_header(res_html, meta, preprocessor.H5, occurence=0)
+        #--------------------TITLE and AUTHOR
+        res_html.write("<div class='dt-row header'>")
+        for lang in lang_ordered:
+            res_html.write("<div class='par dt-cell'>")
+            meta = metas["items"][lang]
+            title = get_meta(meta, preprocessor.TITLE)
+            author = get_meta(meta, preprocessor.AUTHOR)
+            if title:
+                res_html.write("<h1 class='lt-title'>" + title + "</h1>")
+            if author:
+                res_html.write("<h1 class='lt-author'>" + author + "</h1>")
+            res_html.write("</div>")
+        res_html.write("</div>")
 
-        # --------------------PARAGRAPHS
-        for p_from, p_to in zip(paragraphs_from, paragraphs_to):
+        next_mark, next_meta_par_id = get_next_meta_par_id(metas)
 
-            if p_from == H1_MARK:
-                write_header(res_html, meta, preprocessor.H1,
-                             occurence=p_to, add_divider=True)
-            elif p_from == H2_MARK:
-                write_header(res_html, meta, preprocessor.H2,
-                             occurence=p_to, add_divider=True)
-            elif p_from == H3_MARK:
-                write_header(res_html, meta, preprocessor.H3,
-                             occurence=p_to, add_divider=True)
-            elif p_from == H4_MARK:
-                write_header(res_html, meta, preprocessor.H4,
-                             occurence=p_to, add_divider=True)
-            elif p_from == H5_MARK:
-                write_header(res_html, meta, preprocessor.H5,
-                             occurence=p_to, add_divider=True)
+        for actual_paragraphs_id in range(min_par_len):
+            real_par_id = delimeters[actual_paragraphs_id]
+
+            while next_meta_par_id <= real_par_id:
+                _ = write_next_polyheader(res_html, next_mark, metas, lang_ordered)
+                next_mark, next_meta_par_id = get_next_meta_par_id(metas)
+
+            res_html.write("<div class='dt-row'>")
+            for lang in lang_ordered:
+
+                res_html.write(f"<div class='par dt-cell'>  ")
+
+                for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
+                    res_html.write(
+                        f"<span class='s s{k%sent_cycle}'>{sent}</span>")
 
-            elif p_from == DIVIDER_MARK:
-                res_html.write("<div class='dt-row header'>")
-                res_html.write(
-                    "<div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div><div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div>")
                 res_html.write("</div>")
-            else:
-                res_html.write("<div class='dt-row'><div class='par dt-cell'>")
-                for i, sent in enumerate(p_from):
-                    res_html.write(
-                        f"<span class='sent sent-{i%sent_cycle}'>{sent}</span>")
-                res_html.write("</div><div class='par dt-cell'>")
-                for i, sent in enumerate(p_to):
-                    res_html.write(
-                        f"<span class='sent sent-{i%sent_cycle}'>{sent}</span>")
-                res_html.write("</div></div>")
 
-        res_html.write("</div>")
-        res_html.write("</body></html>")
+            res_html.write("</div>")
 
-
-def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, template, styles=[], par_amount=0):
-    """Generate multiligual html preview"""
-    # ensure path is existed
-    langs_count = len(lang_ordered)   
-
-    if template in STYLES:
-        css = generate_css(STYLES[template], cols=langs_count)
-        sent_cycle = len(STYLES[template])
-    elif template == "custom" and styles:
-        css = generate_css(styles, cols=langs_count)
-        sent_cycle = len(styles)
-    else:
-        css = generate_css([], cols=langs_count)
-        template = "simple"
-        sent_cycle = 4
-
-    res_html = ""
-    # --------------------BOOK
-    res_html += "<div class='dt cont'>"
-
-    #--------------------TITLE and AUTHOR
-    res_html += "<div class='dt-row header'>"
-    for lang in lang_ordered:
-        res_html += "<div class='par dt-cell'>"
-        meta = metas["items"][lang]
-        title = get_meta(meta, preprocessor.TITLE)
-        author = get_meta(meta, preprocessor.AUTHOR)
-        if title:
-            res_html += "<h1 class='lt-title'>" + title + "</h1>"
-        if author:
-            res_html += "<h1 class='lt-author'>" + author + "</h1>"
-        res_html += "</div>"
-    res_html += "</div>"
-
-    # --------------------DIVIDER
-    res_html += "<div class='dt-row header'>"
-    for _ in range(langs_count):
-        res_html += "<div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div>"
-    res_html += "</div>"
-
-    # --------------------PARAGRAPHS
-    next_mark, next_meta_par_id = get_next_meta_par_id(metas)
-    min_par_len = min([len(paragraphs[x]) for x in paragraphs])
-    if par_amount > 0:
-        min_par_len = min(min_par_len, par_amount)
-
-    for actual_paragraphs_id in range(min_par_len):
-        real_par_id = delimeters[actual_paragraphs_id]
-
-        while next_meta_par_id <= real_par_id:
-            res_html = write_next_polyheader(res_html, next_mark, metas, lang_ordered, add_string=True)
+        while next_mark:
+            _ = write_next_polyheader(res_html, next_mark, metas, lang_ordered)
             next_mark, next_meta_par_id = get_next_meta_par_id(metas)
 
-        res_html += "<div class='dt-row'>"
-        for lang in lang_ordered:
-
-            res_html += f"<div class='par dt-cell'>"
-
-            for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
-                res_html += f"<span class='sent sent-{k%sent_cycle} {template}'>{sent}</span>"
-
-            res_html += "</div>"
-
-        res_html += "</div>"
-
-    # --------------------END BOOK
-    res_html += "</div>"
-    return res_html
+        # --------------------END BOOK
+        res_html.write(f"</div>{HTML_FOOTER}")
+        res_html.write("</body></html>")
 
 
 def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, template, styles=[]):
@@ -674,19 +524,27 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
     with open(output_path, "w", encoding="utf8") as res_html:
         # --------------------HEAD
         res_html.write(f"""
-        <html><head>
-            <link rel="stylesheet" href="main.css">
-            <link rel="preconnect" href="https://fonts.gstatic.com">
-            <link href="https://fonts.googleapis.com/css2?family=Noto+Serif:wght@400&display=swap" rel="stylesheet">
-            <link href="https://fonts.googleapis.com/css2?family=Oswald:wght@300;400&display=swap" rel="stylesheet">
-            <title>Lingtrain Magic Book</title>
-            {css}
-        </head>
-        <body>""")
+<html><head>
+    <link rel="preconnect" href="https://fonts.gstatic.com">
+    <link href="https://fonts.googleapis.com/css2?family=Noto+Serif:wght@400&display=swap" rel="stylesheet">
+    <link href="https://fonts.googleapis.com/css2?family=Oswald:wght@300;400&display=swap" rel="stylesheet">
+    <link href="https://fonts.googleapis.com/css2?family=Josefin+Sans&display=swap" rel="stylesheet">
+    <title>Lingtrain Magic Book</title>
+    {css}
+</head>
+<body>
+<div class="lt-header">🚀 created in lingtrain alignment studio</div>""")
 
         # --------------------BOOK
         res_html.write("<div class='dt cont'>")
 
+        # --------------------DIVIDER
+        res_html.write("<div class='dt-row header'>")
+        for _ in range(langs_count):
+            res_html.write(
+                f"<div class='dt-cell divider'><img class='divider-img' src='{DIVIDER_URL}'/></div>")
+        res_html.write("</div>")
+
         #--------------------TITLE and AUTHOR
         res_html.write("<div class='dt-row header'>")
         for lang in lang_ordered:
@@ -701,13 +559,6 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
             res_html.write("</div>")
         res_html.write("</div>")
 
-        # --------------------DIVIDER
-        res_html.write("<div class='dt-row header'>")
-        for _ in range(langs_count):
-            res_html.write(
-                "<div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div>")
-        res_html.write("</div>")
-
         # --------------------PARAGRAPHS
         next_mark, next_meta_par_id = get_next_meta_par_id(metas)
         min_par_len = min([len(paragraphs[x]) for x in paragraphs])
@@ -726,17 +577,92 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
 
                 for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
                     res_html.write(
-                        f"<span class='sent sent-{k%sent_cycle}'>{sent}</span>")
+                        f"<span class='s s{k%sent_cycle}'>{sent}</span>")
 
                 res_html.write("</div>")
 
             res_html.write("</div>")
 
+        while next_mark:
+            _ = write_next_polyheader(res_html, next_mark, metas, lang_ordered)
+            next_mark, next_meta_par_id = get_next_meta_par_id(metas)
+
         # --------------------END BOOK
-        res_html.write("</div>")
+        res_html.write(f"</div>{HTML_FOOTER}")
         res_html.write("</body></html>")
 
 
+def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, template, styles=[], par_amount=0):
+    """Generate multiligual html preview"""
+    # ensure path is existed
+    langs_count = len(lang_ordered)
+
+    if template in STYLES:
+        css = generate_css(STYLES[template], cols=langs_count)
+        sent_cycle = len(STYLES[template])
+    elif template == "custom" and styles:
+        css = generate_css(styles, cols=langs_count)
+        sent_cycle = len(styles)
+    else:
+        css = generate_css([], cols=langs_count)
+        template = "simple"
+        sent_cycle = 4
+
+    res_html = ""
+    # --------------------BOOK
+    res_html += "<div class='dt cont'>"
+
+
+    # --------------------DIVIDER
+    res_html += "<div class='dt-row header'>"
+    for _ in range(langs_count):
+        res_html += f"<div class='dt-cell divider'><img class='divider-img' src='{DIVIDER_URL}'/></div>"
+    res_html += "</div>"
+
+    #--------------------TITLE and AUTHOR
+    res_html += "<div class='dt-row header'>"
+    for lang in lang_ordered:
+        res_html += "<div class='par dt-cell'>"
+        meta = metas["items"][lang]
+        title = get_meta(meta, preprocessor.TITLE)
+        author = get_meta(meta, preprocessor.AUTHOR)
+        if title:
+            res_html += "<h1 class='lt-title'>" + title + "</h1>"
+        if author:
+            res_html += "<h1 class='lt-author'>" + author + "</h1>"
+        res_html += "</div>"
+    res_html += "</div>"
+
+    # --------------------PARAGRAPHS
+    next_mark, next_meta_par_id = get_next_meta_par_id(metas)
+    min_par_len = min([len(paragraphs[x]) for x in paragraphs])
+    if par_amount > 0:
+        min_par_len = min(min_par_len, par_amount)
+
+    for actual_paragraphs_id in range(min_par_len):
+        real_par_id = delimeters[actual_paragraphs_id]
+
+        while next_meta_par_id <= real_par_id:
+            res_html = write_next_polyheader(res_html, next_mark, metas, lang_ordered, add_string=True)
+            next_mark, next_meta_par_id = get_next_meta_par_id(metas)
+
+        res_html += "<div class='dt-row'>"
+        for lang in lang_ordered:
+
+            res_html += f"<div class='par dt-cell'>"
+
+            for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
+                res_html += f"<span class='s s{k%sent_cycle} {template}'>{sent}</span>"
+
+            res_html += "</div>"
+
+        res_html += "</div>"
+
+    # --------------------END BOOK
+    res_html += "</div>"
+    return res_html
+
+
 def get_next_meta_par_id(metas):
     lang_code = metas["main_lang_code"]
     meta = metas["items"][lang_code]
@@ -771,7 +697,7 @@ def write_next_polyheader(writer, next_mark, metas_dict, lang_ordered, add_strin
         elif next_mark == preprocessor.IMAGE:
             el = f"<div class='par dt-cell text-center'><img class='lt-image' src='{val[0]}'/></div>"
         elif next_mark == preprocessor.DIVIDER:
-            el = "<div class='dt-cell divider'><img class='divider-img' src='https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png'/></div>"
+            el = f"<div class='dt-cell divider'><img class='divider-img' src='{DIVIDER_URL}'/></div>"
         else:
             el = f"<div class='par dt-cell'><{HEADER_HTML_MAPPING[next_mark]}>{val[0]}</{HEADER_HTML_MAPPING[next_mark]}></div>"
 
@@ -789,30 +715,6 @@ def write_next_polyheader(writer, next_mark, metas_dict, lang_ordered, add_strin
     return writer
 
 
-def write_header(writer, meta, mark, occurence, add_divider=False):
-    meta_from = get_meta_from(meta, mark, occurence)
-    meta_to = get_meta_to(meta, mark, occurence)
-    if not meta_to:
-        meta_to = meta_from
-
-    if meta_from:
-        if add_divider:
-            writer.write("<div class='dt-row header'>")
-            writer.write(
-                "<div class='dt-cell divider'></div><div class='dt-cell divider'></div>")
-            writer.write("</div>")
-        # left
-        writer.write("<div class='dt-row header'><div class='par dt-cell'>")
-        writer.write(f'<{HEADER_HTML_MAPPING[mark]}>' +
-                     meta_from[0] + f'</{HEADER_HTML_MAPPING[mark]}>')
-        writer.write("</div>")
-        # right
-        writer.write("<div class='par dt-cell'>")
-        writer.write(f'<{HEADER_HTML_MAPPING[mark]}>' + get_meta_to(meta,
-                                                                    mark, occurence)[0] + f'</{HEADER_HTML_MAPPING[mark]}>')
-        writer.write("</div></div>")
-
-
 def prepare_meta(meta, direction):
     """Take all meta for an exact direction"""
     res = dict()
@@ -850,7 +752,7 @@ def generate_css(styles, cols=2):
     special = ""
     for i, s in enumerate(styles):
         style = json.loads(s)
-        special += f".sent-{i} {{\n"
+        special += f".s{i} {{\n"
         for rule in style:
             special += f"{rule}: {style[rule]};"
         special += "\n}"
@@ -869,6 +771,12 @@ HEADER_HTML_MAPPING = {
     preprocessor.H5: "h4"
 }
 
+# DIVIDER_URL = "https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png"
+DIVIDER_URL = "https://habrastorage.org/webt/q9/1t/cy/q91tcypgjnrrsmrfcviyzzdvfsk.png"
+
+HTML_FOOTER = """
+<div class="lt-footer">🚀 created in lingtrain alignment studio</div>"""
+
 CSS_TEMPLATE = """<style type="text/css">
 %GENERAL%
 %SPECIAL%
@@ -894,10 +802,6 @@ CSS_GENERAL = """
     }
 }
 
-# body {
-#     background: #fbfbf8;
-# }
-
 h1, h2, h3, h4, h5 {
     font-family: 'Oswald', cursive;
 }
@@ -912,11 +816,15 @@ h2 {
 }
 
 h3 {
-    font-weight: 300;
+    font-weight: 400;
 }
 
 .cont {
     margin: 0 150px;
+    # background: #fcfcfc;
+    # border: 1px solid #efefef;
+    # border-radius: 10px;
+    # overflow: hidden;
 }
 
 @media print {
@@ -928,7 +836,6 @@ h3 {
 .dt {
     display: table;
 }
-
 .dt-row {
     display: table-row;
 }
@@ -960,7 +867,8 @@ h3 {
 }
 
 .divider-img {
-    width: 100px;
+    width: 50px;
+    height: 50px;
 }
 
 .flag-img {
@@ -973,9 +881,10 @@ h3 {
     display: inline-block;
 }
 
-.sent {
+.s {
     padding: 0 6px;
 }
+
 .text-center {
     text-align: center;
 }
@@ -991,23 +900,53 @@ h3 {
 }
 .lt-image {
     max-height: 300px;
+    max-width: 800px;
 }
 .lt-title {
-    font-size: 42px;
+    font-size: 52px;
+    text-align: center;
 }
 .lt-author {
     font-size: 40px;
+    text-align: center;
 }
 
 @media print {
     .lt-title {
-        font-size: 36px;
+        font-size: 32px;
     }
     .lt-author {
-        font-size: 36px;
+        font-size: 28px;
+    }
+    .lt-image {
+        max-height: 260px;
+        max-width: 400px;
+    }
+    .divider-img {
+        height: 30px;
+        width: 30px;
     }
 }
 
+.lt-footer {
+    padding: 20px 0;
+    margin-top: 30px;
+    text-align:center;
+    font-family: 'Josefin Sans', sans-serif;
+    text-transform: capitalize;
+    color: white;
+    background: cornflowerblue;
+}
+
+.lt-header {
+    padding: 20px 0;
+    margin-bottom: 30px;
+    text-align:center;
+    font-family: 'Josefin Sans', sans-serif;
+    text-transform: capitalize;
+    color: white;
+    background: cornflowerblue;
+}
 """
 
 
@@ -1019,9 +958,9 @@ STYLES = {
         '{"background": "#fefbd6", "color": "black"}'
     ],
     "pastel_start": [
-        '{"background": "linear-gradient(90deg, #cfefd7 0px, #ffffff00 150px)"}',
-        '{"background": "linear-gradient(90deg, #fadce2 0px, #ffffff00 150px)"}',
-        '{"background": "linear-gradient(90deg, #cce7eb 0px, #ffffff00 150px)"}',
-        '{"background": "linear-gradient(90deg, #fefbd6 0px, #ffffff00 150px)"}'
+        '{"background": "linear-gradient(90deg, #cfefd7 0px, #ffffff00 150px)", "border-radius": "15px"}',
+        '{"background": "linear-gradient(90deg, #fadce2 0px, #ffffff00 150px)", "border-radius": "15px"}',
+        '{"background": "linear-gradient(90deg, #cce7eb 0px, #ffffff00 150px)", "border-radius": "15px"}',
+        '{"background": "linear-gradient(90deg, #fefbd6 0px, #ffffff00 150px)", "border-radius": "15px"}'
     ]
 }
